/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * Copyright by the Board of Trustees of the University of Illinois.         *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of HDF5.  The full HDF5 copyright notice, including     *
 * terms governing use, modification, and redistribution, is contained in    *
 * the files COPYING and Copyright.html.  COPYING can be found at the root   *
 * of the source code distribution tree; Copyright.html can be found at the  *
 * root level of an installed copy of the electronic HDF5 document set and   *
 * is linked from the top-level documents page.  It can also be found at     *
 * http://hdfgroup.org/HDF5/doc/Copyright.html.  If you do not have          *
 * access to either file, you may request a copy from help@hdfgroup.org.     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/*
 *  For details of the HDF libraries, see the HDF Documentation at:
 *    http://hdfdfgroup.org/HDF5/doc/
 *
 */

#ifndef _Included_h5jni
#define _Included_h5jni

#include <jni.h>
#include "H5version.h"
#include <string.h>

#ifdef H5_HAVE_WIN32_API
/* The following two defines must be before any windows headers are included */
#define WIN32_LEAN_AND_MEAN    /* Exclude rarely-used stuff from Windows headers */
#define NOGDI                  /* Exclude Graphic Display Interface macros */

#ifdef H5_HAVE_WINSOCK2_H
#include <winsock2.h>
#endif

#ifdef H5_HAVE_THREADSAFE
#include <process.h>            /* For _beginthread() */
#endif

#include <windows.h>
#include <direct.h>         /* For _getcwd() */

#endif /*H5_HAVE_WIN32_API*/

/*
 * HDF Boolean type.
 */
#ifndef FALSE
#   define FALSE 0
#endif
#ifndef TRUE
#   define TRUE 1
#endif

/*
 * Redefine all the POSIX functions.  We should never see a POSIX
 * function (or any other non-HDF5 function) in the source!
 */

/* Put all platform-specific definitions in the following file */
/* so that the following definitions are platform free. */
#include "H5win32defs.h"  /* For Windows-specific definitions */

#ifndef HDabort
    #define HDabort()    abort()
#endif /* HDabort */
#ifndef HDabs
    #define HDabs(X)    abs(X)
#endif /* HDabs */
#ifndef HDaccess
    #define HDaccess(F,M)    access(F, M)
#endif /* HDaccess */
#ifndef HDacos
    #define HDacos(X)    acos(X)
#endif /* HDacos */
#ifndef HDalarm
    #ifdef H5_HAVE_ALARM
        #define HDalarm(N)              alarm(N)
    #else /* H5_HAVE_ALARM */
        #define HDalarm(N)              (0)
    #endif /* H5_HAVE_ALARM */
#endif /* HDalarm */
#ifndef HDasctime
    #define HDasctime(T)    asctime(T)
#endif /* HDasctime */
#ifndef HDasin
    #define HDasin(X)    asin(X)
#endif /* HDasin */
#ifndef HDasprintf
    #define HDasprintf    asprintf /*varargs*/
#endif /* HDasprintf */
#ifndef HDassert
    #define HDassert(X)    assert(X)
#endif /* HDassert */
#ifndef HDatan
    #define HDatan(X)    atan(X)
#endif /* HDatan */
#ifndef HDatan2
    #define HDatan2(X,Y)    atan2(X,Y)
#endif /* HDatan2 */
#ifndef HDatexit
    #define HDatexit(F)    atexit(F)
#endif /* HDatexit */
#ifndef HDatof
    #define HDatof(S)    atof(S)
#endif /* HDatof */
#ifndef HDatoi
    #define HDatoi(S)    atoi(S)
#endif /* HDatoi */
#ifndef HDatol
    #define HDatol(S)    atol(S)
#endif /* HDatol */
#ifndef HDbsearch
    #define HDbsearch(K,B,N,Z,F)  bsearch(K,B,N,Z,F)
#endif /* HDbsearch */
#ifndef HDcalloc
    #define HDcalloc(N,Z)    calloc(N,Z)
#endif /* HDcalloc */
#ifndef HDceil
    #define HDceil(X)    ceil(X)
#endif /* HDceil */
#ifndef HDcfgetispeed
    #define HDcfgetispeed(T)  cfgetispeed(T)
#endif /* HDcfgetispeed */
#ifndef HDcfgetospeed
    #define HDcfgetospeed(T)  cfgetospeed(T)
#endif /* HDcfgetospeed */
#ifndef HDcfsetispeed
    #define HDcfsetispeed(T,S)  cfsetispeed(T,S)
#endif /* HDcfsetispeed */
#ifndef HDcfsetospeed
    #define HDcfsetospeed(T,S)  cfsetospeed(T,S)
#endif /* HDcfsetospeed */
#ifndef HDchdir
    #define HDchdir(S)    chdir(S)
#endif /* HDchdir */
#ifndef HDchmod
    #define HDchmod(S,M)    chmod(S,M)
#endif /* HDchmod */
#ifndef HDchown
    #define HDchown(S,O,G)    chown(S,O,G)
#endif /* HDchown */
#ifndef HDclearerr
    #define HDclearerr(F)    clearerr(F)
#endif /* HDclearerr */
#ifndef HDclock
    #define HDclock()    clock()
#endif /* HDclock */
#ifndef HDclose
    #define HDclose(F)    close(F)
#endif /* HDclose */
#ifndef HDclosedir
    #define HDclosedir(D)    closedir(D)
#endif /* HDclosedir */
#ifndef HDcos
    #define HDcos(X)    cos(X)
#endif /* HDcos */
#ifndef HDcosh
    #define HDcosh(X)    cosh(X)
#endif /* HDcosh */
#ifndef HDcreat
    #define HDcreat(S,M)    creat(S,M)
#endif /* HDcreat */
#ifndef HDctermid
    #define HDctermid(S)    ctermid(S)
#endif /* HDctermid */
#ifndef HDctime
    #define HDctime(T)    ctime(T)
#endif /* HDctime */
#ifndef HDcuserid
    #define HDcuserid(S)    cuserid(S)
#endif /* HDcuserid */
#ifndef HDdifftime
    #ifdef H5_HAVE_DIFFTIME
        #define HDdifftime(X,Y)    difftime(X,Y)
    #else /* H5_HAVE_DIFFTIME */
        #define HDdifftime(X,Y)    ((double)(X)-(double)(Y))
    #endif /* H5_HAVE_DIFFTIME */
#endif /* HDdifftime */
#ifndef HDdiv
    #define HDdiv(X,Y)    div(X,Y)
#endif /* HDdiv */
#ifndef HDdup
    #define HDdup(F)    dup(F)
#endif /* HDdup */
#ifndef HDdup2
    #define HDdup2(F,I)    dup2(F,I)
#endif /* HDdup2 */
/* execl() variable arguments */
/* execle() variable arguments */
/* execlp() variable arguments */
#ifndef HDexecv
    #define HDexecv(S,AV)    execv(S,AV)
#endif /* HDexecv */
#ifndef HDexecve
    #define HDexecve(S,AV,E)  execve(S,AV,E)
#endif /* HDexecve */
#ifndef HDexecvp
    #define HDexecvp(S,AV)    execvp(S,AV)
#endif /* HDexecvp */
#ifndef HDexit
    #define HDexit(N)    exit(N)
#endif /* HDexit */
#ifndef HD_exit
    #define HD_exit(N)    _exit(N)
#endif /* HD_exit */
#ifndef HDexp
    #define HDexp(X)    exp(X)
#endif /* HDexp */
#ifndef HDfabs
    #define HDfabs(X)    fabs(X)
#endif /* HDfabs */
/* use ABS() because fabsf() fabsl() are not common yet. */
#ifndef HDfabsf
    #define HDfabsf(X)    ABS(X)
#endif /* HDfabsf */
#ifndef HDfabsl
    #define HDfabsl(X)    ABS(X)
#endif /* HDfabsl */
#ifndef HDfclose
    #define HDfclose(F)    fclose(F)
#endif /* HDfclose */
/* fcntl() variable arguments */
#ifndef HDfdopen
    #define HDfdopen(N,S)    fdopen(N,S)
#endif /* HDfdopen */
#ifndef HDfeof
    #define HDfeof(F)    feof(F)
#endif /* HDfeof */
#ifndef HDferror
    #define HDferror(F)    ferror(F)
#endif /* HDferror */
#ifndef HDfflush
    #define HDfflush(F)    fflush(F)
#endif /* HDfflush */
#ifndef HDfgetc
    #define HDfgetc(F)    fgetc(F)
#endif /* HDfgetc */
#ifndef HDfgetpos
    #define HDfgetpos(F,P)    fgetpos(F,P)
#endif /* HDfgetpos */
#ifndef HDfgets
    #define HDfgets(S,N,F)    fgets(S,N,F)
#endif /* HDfgets */
#ifndef HDfileno
    #define HDfileno(F)    fileno(F)
#endif /* HDfileno */
#ifndef HDfloor
    #define HDfloor(X)    floor(X)
#endif /* HDfloor */
#ifndef HDfmod
    #define HDfmod(X,Y)    fmod(X,Y)
#endif /* HDfmod */
#ifndef HDfopen
    #define HDfopen(S,M)    fopen(S,M)
#endif /* HDfopen */
#ifndef HDfork
    #define HDfork()    fork()
#endif /* HDfork */
#ifndef HDfpathconf
    #define HDfpathconf(F,N)  fpathconf(F,N)
#endif /* HDfpathconf */
H5_DLL int HDfprintf (FILE *stream, const char *fmt, ...);
#ifndef HDfputc
    #define HDfputc(C,F)    fputc(C,F)
#endif /* HDfputc */
#ifndef HDfputs
    #define HDfputs(S,F)    fputs(S,F)
#endif /* HDfputs */
#ifndef HDfread
    #define HDfread(M,Z,N,F)  fread(M,Z,N,F)
#endif /* HDfread */
#ifndef HDfree
    #define HDfree(M)    free(M)
#endif /* HDfree */
#ifndef HDfreopen
    #define HDfreopen(S,M,F)  freopen(S,M,F)
#endif /* HDfreopen */
#ifndef HDfrexp
    #define HDfrexp(X,N)    frexp(X,N)
#endif /* HDfrexp */
/* Check for Cray-specific 'frexpf()' and 'frexpl()' routines */
#ifndef HDfrexpf
    #ifdef H5_HAVE_FREXPF
        #define HDfrexpf(X,N)    frexpf(X,N)
    #else /* H5_HAVE_FREXPF */
        #define HDfrexpf(X,N)    frexp(X,N)
    #endif /* H5_HAVE_FREXPF */
#endif /* HDfrexpf */
#ifndef HDfrexpl
    #ifdef H5_HAVE_FREXPL
        #define HDfrexpl(X,N)    frexpl(X,N)
    #else /* H5_HAVE_FREXPL */
        #define HDfrexpl(X,N)    frexp(X,N)
    #endif /* H5_HAVE_FREXPL */
#endif /* HDfrexpl */
/* fscanf() variable arguments */
#ifndef HDfseek
    #define HDfseek(F,O,W)  fseeko(F,O,W)
#endif /* HDfseek */
#ifndef HDfsetpos
    #define HDfsetpos(F,P)    fsetpos(F,P)
#endif /* HDfsetpos */
#ifndef HDfstat
    #define HDfstat(F,B)        fstat(F,B)
#endif /* HDfstat */
#ifndef HDlstat
    #define HDlstat(S,B)    lstat(S,B)
#endif /* HDlstat */
#ifndef HDstat
    #define HDstat(S,B)    stat(S,B)
#endif /* HDstat */

#ifndef H5_HAVE_WIN32_API
/* These definitions differ in Windows and are defined in
 * H5win32defs for that platform.
 */
typedef struct stat         h5_stat_t;
typedef off_t               h5_stat_size_t;
#define HDoff_t             off_t
#endif /* H5_HAVE_WIN32_API */

#define H5_SIZEOF_H5_STAT_SIZE_T H5_SIZEOF_OFF_T

#ifndef HDftell
    #define HDftell(F)    ftello(F)
#endif /* HDftell */
#ifndef HDftruncate
    #define HDftruncate(F,L)        ftruncate(F,L)
#endif /* HDftruncate */
#ifndef HDfwrite
    #define HDfwrite(M,Z,N,F)  fwrite(M,Z,N,F)
#endif /* HDfwrite */
#ifndef HDgetc
    #define HDgetc(F)    getc(F)
#endif /* HDgetc */
#ifndef HDgetchar
    #define HDgetchar()    getchar()
#endif /* HDgetchar */
#ifndef HDgetcwd
    #define HDgetcwd(S,Z)    getcwd(S,Z)
#endif /* HDgetcwd */
#ifndef HDgetdcwd
    #define HDgetdcwd(D,S,Z)  getcwd(S,Z)
#endif /* HDgetdcwd */
#ifndef HDgetdrive
    #define HDgetdrive()    0
#endif /* HDgetdrive */
#ifndef HDgetegid
    #define HDgetegid()    getegid()
#endif /* HDgetegid() */
#ifndef HDgetenv
    #define HDgetenv(S)    getenv(S)
#endif /* HDgetenv */
#ifndef HDgeteuid
    #define HDgeteuid()    geteuid()
#endif /* HDgeteuid */
#ifndef HDgetgid
    #define HDgetgid()    getgid()
#endif /* HDgetgid */
#ifndef HDgetgrgid
    #define HDgetgrgid(G)    getgrgid(G)
#endif /* HDgetgrgid */
#ifndef HDgetgrnam
    #define HDgetgrnam(S)    getgrnam(S)
#endif /* HDgetgrnam */
#ifndef HDgetgroups
    #define HDgetgroups(Z,G)  getgroups(Z,G)
#endif /* HDgetgroups */
#ifndef HDgethostname
    #define HDgethostname(N,L)    gethostname(N,L)
#endif /* HDgetlogin */
#ifndef HDgetlogin
    #define HDgetlogin()    getlogin()
#endif /* HDgetlogin */
#ifndef HDgetpgrp
    #define HDgetpgrp()    getpgrp()
#endif /* HDgetpgrp */
#ifndef HDgetpid
    #define HDgetpid()    getpid()
#endif /* HDgetpid */
#ifndef HDgetppid
    #define HDgetppid()    getppid()
#endif /* HDgetppid */
#ifndef HDgetpwnam
    #define HDgetpwnam(S)    getpwnam(S)
#endif /* HDgetpwnam */
#ifndef HDgetpwuid
    #define HDgetpwuid(U)    getpwuid(U)
#endif /* HDgetpwuid */
#ifndef HDgetrusage
    #define HDgetrusage(X,S)  getrusage(X,S)
#endif /* HDgetrusage */
#ifndef HDgets
    #define HDgets(S)    gets(S)
#endif /* HDgets */
#ifndef HDgettimeofday
    #define HDgettimeofday(S,P)  gettimeofday(S,P)
#endif /* HDgettimeofday */
#ifndef HDgetuid
    #define HDgetuid()    getuid()
#endif /* HDgetuid */
#ifndef HDgmtime
    #define HDgmtime(T)    gmtime(T)
#endif /* HDgmtime */
#ifndef HDisalnum
    #define HDisalnum(C)    isalnum((int)(C)) /*cast for solaris warning*/
#endif /* HDisalnum */
#ifndef HDisalpha
    #define HDisalpha(C)    isalpha((int)(C)) /*cast for solaris warning*/
#endif /* HDisalpha */
#ifndef HDisatty
    #define HDisatty(F)    isatty(F)
#endif /* HDisatty */
#ifndef HDiscntrl
    #define HDiscntrl(C)    iscntrl((int)(C)) /*cast for solaris warning*/
#endif /* HDiscntrl */
#ifndef HDisdigit
    #define HDisdigit(C)    isdigit((int)(C)) /*cast for solaris warning*/
#endif /* HDisdigit */
#ifndef HDisgraph
    #define HDisgraph(C)    isgraph((int)(C)) /*cast for solaris warning*/
#endif /* HDisgraph */
#ifndef HDislower
    #define HDislower(C)    islower((int)(C)) /*cast for solaris warning*/
#endif /* HDislower */
#ifndef HDisprint
    #define HDisprint(C)    isprint((int)(C)) /*cast for solaris warning*/
#endif /* HDisprint */
#ifndef HDispunct
    #define HDispunct(C)    ispunct((int)(C)) /*cast for solaris warning*/
#endif /* HDispunct */
#ifndef HDisspace
    #define HDisspace(C)    isspace((int)(C)) /*cast for solaris warning*/
#endif /* HDisspace */
#ifndef HDisupper
    #define HDisupper(C)    isupper((int)(C)) /*cast for solaris warning*/
#endif /* HDisupper */
#ifndef HDisxdigit
    #define HDisxdigit(C)    isxdigit((int)(C)) /*cast for solaris warning*/
#endif /* HDisxdigit */
#ifndef HDkill
    #define HDkill(P,S)    kill(P,S)
#endif /* HDkill */
#ifndef HDlabs
    #define HDlabs(X)    labs(X)
#endif /* HDlabs */
#ifndef HDldexp
    #define HDldexp(X,N)    ldexp(X,N)
#endif /* HDldexp */
#ifndef HDldiv
    #define HDldiv(X,Y)    ldiv(X,Y)
#endif /* HDldiv */
#ifndef HDlink
    #define HDlink(OLD,NEW)    link(OLD,NEW)
#endif /* HDlink */
#ifndef HDlocaleconv
    #define HDlocaleconv()    localeconv()
#endif /* HDlocaleconv */
#ifndef HDlocaltime
    #define HDlocaltime(T)    localtime(T)
#endif /* HDlocaltime */
#ifndef HDlog
    #define HDlog(X)    log(X)
#endif /* HDlog */
#ifndef HDlog10
    #define HDlog10(X)    log10(X)
#endif /* HDlog10 */
#ifndef HDlongjmp
    #define HDlongjmp(J,N)    longjmp(J,N)
#endif /* HDlongjmp */
#ifndef HDlseek
    #define HDlseek(F,O,W)  lseek(F,O,W)
#endif /* HDlseek */
#ifndef HDmalloc
    #define HDmalloc(Z)    malloc(Z)
#endif /* HDmalloc */
#ifndef HDposix_memalign
    #define HDposix_memalign(P,A,Z) posix_memalign(P,A,Z)
#endif /* HDposix_memalign */
#ifndef HDmblen
    #define HDmblen(S,N)    mblen(S,N)
#endif /* HDmblen */
#ifndef HDmbstowcs
    #define HDmbstowcs(P,S,Z)  mbstowcs(P,S,Z)
#endif /* HDmbstowcs */
#ifndef HDmbtowc
    #define HDmbtowc(P,S,Z)    mbtowc(P,S,Z)
#endif /* HDmbtowc */
#ifndef HDmemchr
    #define HDmemchr(S,C,Z)    memchr(S,C,Z)
#endif /* HDmemchr */
#ifndef HDmemcmp
    #define HDmemcmp(X,Y,Z)    memcmp(X,Y,Z)
#endif /* HDmemcmp */
/*
 * The (char*) casts are required for the DEC when optimizations are turned
 * on and the source and/or destination are not aligned.
 */
#ifndef HDmemcpy
    #define HDmemcpy(X,Y,Z)    memcpy((char*)(X),(const char*)(Y),Z)
#endif /* HDmemcpy */
#ifndef HDmemmove
    #define HDmemmove(X,Y,Z)  memmove((char*)(X),(const char*)(Y),Z)
#endif /* HDmemmove */
#ifndef HDmemset
    #define HDmemset(X,C,Z)    memset(X,C,Z)
#endif /* HDmemset */
#ifndef HDmkdir
    #define HDmkdir(S,M)    mkdir(S,M)
#endif /* HDmkdir */
#ifndef HDmkfifo
    #define HDmkfifo(S,M)    mkfifo(S,M)
#endif /* HDmkfifo */
#ifndef HDmktime
    #define HDmktime(T)    mktime(T)
#endif /* HDmktime */
#ifndef HDmodf
    #define HDmodf(X,Y)    modf(X,Y)
#endif /* HDmodf */
#ifndef HDopen
    #ifdef _O_BINARY
        #define HDopen(S,F,M)    open(S,F|_O_BINARY,M)
    #else
        #define HDopen(S,F,M)    open(S,F,M)
    #endif
#endif /* HDopen */
#ifndef HDopendir
    #define HDopendir(S)    opendir(S)
#endif /* HDopendir */
#ifndef HDpathconf
    #define HDpathconf(S,N)    pathconf(S,N)
#endif /* HDpathconf */
#ifndef HDpause
    #define HDpause()    pause()
#endif /* HDpause */
#ifndef HDperror
    #define HDperror(S)    perror(S)
#endif /* HDperror */
#ifndef HDpipe
    #define HDpipe(F)    pipe(F)
#endif /* HDpipe */
#ifndef HDpow
    #define HDpow(X,Y)    pow(X,Y)
#endif /* HDpow */
/* printf() variable arguments */
#ifndef HDputc
    #define HDputc(C,F)    putc(C,F)
#endif /* HDputc*/
#ifndef HDputchar
    #define HDputchar(C)    putchar(C)
#endif /* HDputchar */
#ifndef HDputs
    #define HDputs(S)    puts(S)
#endif /* HDputs */
#ifndef HDqsort
    #define HDqsort(M,N,Z,F)  qsort(M,N,Z,F)
#endif /* HDqsort*/
#ifndef HDraise
    #define HDraise(N)    raise(N)
#endif /* HDraise */

#ifdef H5_HAVE_RAND_R
    #ifndef HDrandom
        #define HDrandom()    HDrand()
    #endif /* HDrandom */
    H5_DLL int HDrand(void);
#elif H5_HAVE_RANDOM
    #ifndef HDrand
        #define HDrand()    random()
    #endif /* HDrand */
    #ifndef HDrandom
        #define HDrandom()    random()
    #endif /* HDrandom */
#else /* H5_HAVE_RANDOM */
    #ifndef HDrand
        #define HDrand()    rand()
    #endif /* HDrand */
    #ifndef HDrandom
        #define HDrandom()    rand()
    #endif /* HDrandom */
#endif /* H5_HAVE_RANDOM */

#ifndef HDread
    #define HDread(F,M,Z)    read(F,M,Z)
#endif /* HDread */
#ifndef HDreaddir
    #define HDreaddir(D)    readdir(D)
#endif /* HDreaddir */
#ifndef HDrealloc
    #define HDrealloc(M,Z)    realloc(M,Z)
#endif /* HDrealloc */
#ifndef HDrealpath
    #define HDrealpath(F1,F2)    realpath(F1,F2)
#endif /* HDrealloc */
#ifndef HDremove
    #define HDremove(S)    remove(S)
#endif /* HDremove */
#ifndef HDrename
    #define HDrename(OLD,NEW)  rename(OLD,NEW)
#endif /* HDrename */
#ifndef HDrewind
    #define HDrewind(F)    rewind(F)
#endif /* HDrewind */
#ifndef HDrewinddir
    #define HDrewinddir(D)    rewinddir(D)
#endif /* HDrewinddir */
#ifndef HDrmdir
    #define HDrmdir(S)    rmdir(S)
#endif /* HDrmdir */
/* scanf() variable arguments */
#ifndef HDsetbuf
    #define HDsetbuf(F,S)    setbuf(F,S)
#endif /* HDsetbuf */
#ifndef HDsetenv
    #define HDsetenv(N,V,O)    setenv(N,V,O)
#endif /* HDsetenv */
#ifndef HDsetgid
    #define HDsetgid(G)    setgid(G)
#endif /* HDsetgid */
#ifndef HDsetjmp
    #define HDsetjmp(J)    setjmp(J)
#endif /* HDsetjmp */
#ifndef HDsetlocale
    #define HDsetlocale(N,S)  setlocale(N,S)
#endif /* HDsetlocale */
#ifndef HDsetpgid
    #define HDsetpgid(P,PG)    setpgid(P,PG)
#endif /* HDsetpgid */
#ifndef HDsetsid
    #define HDsetsid()    setsid()
#endif /* HDsetsid */
#ifndef HDsetuid
    #define HDsetuid(U)    setuid(U)
#endif /* HDsetuid */
#ifndef HDsetvbuf
    #define HDsetvbuf(F,S,M,Z)  setvbuf(F,S,M,Z)
#endif /* HDsetvbuf */
#ifndef HDsigaddset
    #define HDsigaddset(S,N)  sigaddset(S,N)
#endif /* HDsigaddset */
#ifndef HDsigdelset
    #define HDsigdelset(S,N)  sigdelset(S,N)
#endif /* HDsigdelset */
#ifndef HDsigemptyset
    #define HDsigemptyset(S)  sigemptyset(S)
#endif /* HDsigemptyset */
#ifndef HDsigfillset
    #define HDsigfillset(S)    sigfillset(S)
#endif /* HDsigfillset */
#ifndef HDsigismember
    #define HDsigismember(S,N)  sigismember(S,N)
#endif /* HDsigismember */
#ifndef HDsiglongjmp
    #define HDsiglongjmp(J,N)  siglongjmp(J,N)
#endif /* HDsiglongjmp */
#ifndef HDsignal
    #define HDsignal(N,F)    signal(N,F)
#endif /* HDsignal */
#ifndef HDsigpending
    #define HDsigpending(S)    sigpending(S)
#endif /* HDsigpending */
#ifndef HDsigprocmask
    #define HDsigprocmask(H,S,O)  sigprocmask(H,S,O)
#endif /* HDsigprocmask */
#ifndef HDsigsetjmp
    #define HDsigsetjmp(J,N)  sigsetjmp(J,N)
#endif /* HDsigsetjmp */
#ifndef HDsigsuspend
    #define HDsigsuspend(S)    sigsuspend(S)
#endif /* HDsigsuspend */
#ifndef HDsin
    #define HDsin(X)    sin(X)
#endif /* HDsin */
#ifndef HDsinh
    #define HDsinh(X)    sinh(X)
#endif /* HDsinh */
#ifndef HDsleep
    #define HDsleep(N)    sleep(N)
#endif /* HDsleep */
#ifndef HDsnprintf
    #define HDsnprintf    snprintf /*varargs*/
#endif /* HDsnprintf */
#ifndef HDsprintf
    #define HDsprintf    sprintf /*varargs*/
#endif /* HDsprintf */
#ifndef HDsqrt
    #define HDsqrt(X)    sqrt(X)
#endif /* HDsqrt */
#ifdef H5_HAVE_RAND_R
    H5_DLL void HDsrand(unsigned int seed);
    #ifndef HDsrandom
        #define HDsrandom(S)    HDsrand(S)
    #endif /* HDsrandom */
#elif H5_HAVE_RANDOM
    #ifndef HDsrand
        #define HDsrand(S)    srandom(S)
    #endif /* HDsrand */
    #ifndef HDsrandom
        #define HDsrandom(S)    srandom(S)
    #endif /* HDsrandom */
#else /* H5_HAVE_RAND_R */
    #ifndef HDsrand
        #define HDsrand(S)    srand(S)
    #endif /* HDsrand */
    #ifndef HDsrandom
        #define HDsrandom(S)    srand(S)
    #endif /* HDsrandom */
#endif /* H5_HAVE_RAND_R */
/* sscanf() variable arguments */

#ifndef HDstrcat
    #define HDstrcat(X,Y)    strcat(X,Y)
#endif /* HDstrcat */
#ifndef HDstrchr
    #define HDstrchr(S,C)    strchr(S,C)
#endif /* HDstrchr */
#ifndef HDstrcmp
    #define HDstrcmp(X,Y)    strcmp(X,Y)
#endif /* HDstrcmp */
#ifndef HDstrcasecmp
    #define HDstrcasecmp(X,Y)       strcasecmp(X,Y)
#endif /* HDstrcasecmp */
#ifndef HDstrcoll
    #define HDstrcoll(X,Y)    strcoll(X,Y)
#endif /* HDstrcoll */
#ifndef HDstrcpy
    #define HDstrcpy(X,Y)    strcpy(X,Y)
#endif /* HDstrcpy */
#ifndef HDstrcspn
    #define HDstrcspn(X,Y)    strcspn(X,Y)
#endif /* HDstrcspn */
#ifndef HDstrerror
    #define HDstrerror(N)    strerror(N)
#endif /* HDstrerror */
#ifndef HDstrftime
    #define HDstrftime(S,Z,F,T)  strftime(S,Z,F,T)
#endif /* HDstrftime */
#ifndef HDstrlen
    #define HDstrlen(S)    strlen(S)
#endif /* HDstrlen */
#ifndef HDstrncat
    #define HDstrncat(X,Y,Z)  strncat(X,Y,Z)
#endif /* HDstrncat */
#ifndef HDstrncmp
    #define HDstrncmp(X,Y,Z)  strncmp(X,Y,Z)
#endif /* HDstrncmp */
#ifndef HDstrncpy
    #define HDstrncpy(X,Y,Z)  strncpy(X,Y,Z)
#endif /* HDstrncpy */
#ifndef HDstrpbrk
    #define HDstrpbrk(X,Y)    strpbrk(X,Y)
#endif /* HDstrpbrk */
#ifndef HDstrrchr
    #define HDstrrchr(S,C)    strrchr(S,C)
#endif /* HDstrrchr */
#ifndef HDstrspn
    #define HDstrspn(X,Y)    strspn(X,Y)
#endif /* HDstrspn */
#ifndef HDstrstr
    #define HDstrstr(X,Y)    strstr(X,Y)
#endif /* HDstrstr */
#ifndef HDstrtod
    #define HDstrtod(S,R)    strtod(S,R)
#endif /* HDstrtod */
#ifndef HDstrtok
    #define HDstrtok(X,Y)    strtok(X,Y)
#endif /* HDstrtok */
#ifndef HDstrtol
    #define HDstrtol(S,R,N)    strtol(S,R,N)
#endif /* HDstrtol */
#ifndef HDstrtoul
    #define HDstrtoul(S,R,N)  strtoul(S,R,N)
#endif /* HDstrtoul */
#ifndef HDstrtoull
    #define HDstrtoull(S,R,N)  strtoull(S,R,N)
#endif /* HDstrtoul */
#ifndef HDstrxfrm
    #define HDstrxfrm(X,Y,Z)  strxfrm(X,Y,Z)
#endif /* HDstrxfrm */
#ifdef H5_HAVE_SYMLINK
    #ifndef HDsymlink
        #define HDsymlink(F1,F2)  symlink(F1,F2)
    #endif /* HDsymlink */
#endif /* H5_HAVE_SYMLINK */
#ifndef HDsysconf
    #define HDsysconf(N)    sysconf(N)
#endif /* HDsysconf */
#ifndef HDsystem
    #define HDsystem(S)    system(S)
#endif /* HDsystem */
#ifndef HDtan
    #define HDtan(X)    tan(X)
#endif /* HDtan */
#ifndef HDtanh
    #define HDtanh(X)    tanh(X)
#endif /* HDtanh */
#ifndef HDtcdrain
    #define HDtcdrain(F)    tcdrain(F)
#endif /* HDtcdrain */
#ifndef HDtcflow
    #define HDtcflow(F,A)    tcflow(F,A)
#endif /* HDtcflow */
#ifndef HDtcflush
    #define HDtcflush(F,N)    tcflush(F,N)
#endif /* HDtcflush */
#ifndef HDtcgetattr
    #define HDtcgetattr(F,T)  tcgetattr(F,T)
#endif /* HDtcgetattr */
#ifndef HDtcgetpgrp
    #define HDtcgetpgrp(F)    tcgetpgrp(F)
#endif /* HDtcgetpgrp */
#ifndef HDtcsendbreak
    #define HDtcsendbreak(F,N)  tcsendbreak(F,N)
#endif /* HDtcsendbreak */
#ifndef HDtcsetattr
    #define HDtcsetattr(F,O,T)  tcsetattr(F,O,T)
#endif /* HDtcsetattr */
#ifndef HDtcsetpgrp
    #define HDtcsetpgrp(F,N)  tcsetpgrp(F,N)
#endif /* HDtcsetpgrp */
#ifndef HDtime
    #define HDtime(T)    time(T)
#endif /* HDtime */
#ifndef HDtimes
    #define HDtimes(T)    times(T)
#endif /* HDtimes*/
#ifndef HDtmpfile
    #define HDtmpfile()    tmpfile()
#endif /* HDtmpfile */
#ifndef HDtmpnam
    #define HDtmpnam(S)    tmpnam(S)
#endif /* HDtmpnam */
#ifndef HDtolower
    #define HDtolower(C)    tolower(C)
#endif /* HDtolower */
#ifndef HDtoupper
    #define HDtoupper(C)    toupper(C)
#endif /* HDtoupper */
#ifndef HDttyname
    #define HDttyname(F)    ttyname(F)
#endif /* HDttyname */
#ifndef HDtzset
    #define HDtzset()    tzset()
#endif /* HDtzset */
#ifndef HDumask
    #define HDumask(N)    umask(N)
#endif /* HDumask */
#ifndef HDuname
    #define HDuname(S)    uname(S)
#endif /* HDuname */
#ifndef HDungetc
    #define HDungetc(C,F)    ungetc(C,F)
#endif /* HDungetc */
#ifndef HDunlink
    #define HDunlink(S)    unlink(S)
#endif /* HDunlink */
#ifndef HDutime
    #define HDutime(S,T)    utime(S,T)
#endif /* HDutime */
#ifndef HDva_arg
    #define HDva_arg(A,T)    va_arg(A,T)
#endif /* HDva_arg */
#ifndef HDva_end
    #define HDva_end(A)    va_end(A)
#endif /* HDva_end */
#ifndef HDva_start
    #define HDva_start(A,P)    va_start(A,P)
#endif /* HDva_start */
#ifndef HDvasprintf
    #define HDvasprintf(RET,FMT,A)  vasprintf(RET,FMT,A)
#endif /* HDvasprintf */
#ifndef HDvfprintf
    #define HDvfprintf(F,FMT,A)  vfprintf(F,FMT,A)
#endif /* HDvfprintf */
#ifndef HDvprintf
    #define HDvprintf(FMT,A)  vprintf(FMT,A)
#endif /* HDvprintf */
#ifndef HDvsprintf
    #define HDvsprintf(S,FMT,A)  vsprintf(S,FMT,A)
#endif /* HDvsprintf */
#ifndef HDvsnprintf
    #define HDvsnprintf(S,N,FMT,A) vsnprintf(S,N,FMT,A)
#endif /* HDvsnprintf */
#ifndef HDwait
    #define HDwait(W)    wait(W)
#endif /* HDwait */
#ifndef HDwaitpid
    #define HDwaitpid(P,W,O)  waitpid(P,W,O)
#endif /* HDwaitpid */
#ifndef HDwcstombs
    #define HDwcstombs(S,P,Z)  wcstombs(S,P,Z)
#endif /* HDwcstombs */
#ifndef HDwctomb
    #define HDwctomb(S,C)    wctomb(S,C)
#endif /* HDwctomb */
#ifndef HDwrite
    #define HDwrite(F,M,Z)    write(F,M,Z)
#endif /* HDwrite */

/*
 * And now for a couple non-Posix functions...  Watch out for systems that
 * define these in terms of macros.
 */
#if !defined strdup && !defined H5_HAVE_STRDUP
extern char *strdup(const char *s);
#endif

#ifndef HDstrdup
    #define HDstrdup(S)     strdup(S)
#endif /* HDstrdup */

#ifdef __cplusplus
  #define ENVPTR (env)
  #define ENVPAR
  #define ENVONLY
  #define CBENVPTR (cbenv)
  #define CBENVPAR
  #define JVMPTR (jvm)
  #define JVMPAR
  #define JVMPAR2
#else /* __cplusplus */
  #define ENVPTR (*env)
  #define ENVPAR env,
  #define ENVONLY env
  #define CBENVPTR (*cbenv)
  #define CBENVPAR cbenv,
  #define JVMPTR (*jvm)
  #define JVMPAR jvm
  #define JVMPAR2 jvm,
#endif /* __cplusplus */

/* Macros for class access */
/* Calling code must define ret_obj as jobject */
#define CALL_CONSTRUCTOR(classname,classsig,args) {                              \
    jclass     cls;                                                              \
    jmethodID  constructor;                                                      \
    cls = ENVPTR->FindClass(ENVPAR (classname));                                 \
    if (cls == 0) {                                                              \
        h5JNIFatalError(env, "JNI error: GetObjectClass\n");                     \
        ret_obj = NULL;                                                          \
    }                                                                            \
    constructor = ENVPTR->GetMethodID(ENVPAR cls, "<init>", (classsig));         \
    if (constructor == 0) {                                                      \
        h5JNIFatalError(env, "JNI error: GetMethodID failed\n");                 \
        ret_obj = NULL;                                                          \
    }                                                                            \
    ret_obj = ENVPTR->NewObjectA(ENVPAR cls, constructor, (args));               \
}


/* Macros for string access */
#define PIN_JAVA_STRING(javastr,localstr,retdefault) {                           \
    jboolean isCopy;                                                             \
    if ((javastr) == NULL) {                                                     \
        h5nullArgument(env, "java string is NULL");                              \
        return (retdefault);                                                     \
    }                                                                            \
    (localstr) = ENVPTR->GetStringUTFChars(ENVPAR (javastr), &isCopy);           \
    if ((localstr) == NULL) {                                                    \
        h5JNIFatalError(env, "local c string is not pinned");                    \
        return (retdefault);                                                     \
    }                                                                            \
}

#define PIN_JAVA_STRING0(javastr,localstr) {                                     \
    jboolean isCopy;                                                             \
    if ((javastr) == NULL) {                                                     \
        h5nullArgument(env, "java string is NULL");                              \
        return;                                                                  \
    }                                                                            \
    (localstr) = ENVPTR->GetStringUTFChars(ENVPAR (javastr), &isCopy);           \
    if ((localstr) == NULL) {                                                    \
        h5JNIFatalError(env, "local c string is not pinned");                    \
        return;                                                                  \
    }                                                                            \
}

#define UNPIN_JAVA_STRING(javastr,localstr) {                                      \
     ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));                \
}

#define PIN_JAVA_STRING_TWO(javastr,localstr,java2str,local2str,retdefault) {    \
    jboolean isCopy;                                                             \
    if ((javastr) == NULL) {                                                     \
        h5nullArgument(env, "java string is NULL");                              \
        return (retdefault);                                                     \
    }                                                                            \
    if ((java2str) == NULL) {                                                    \
        h5nullArgument(env, "second java string is NULL");                       \
        return (retdefault);                                                     \
    }                                                                            \
    (localstr) = ENVPTR->GetStringUTFChars(ENVPAR (javastr), &isCopy);           \
    if ((localstr) == NULL) {                                                    \
        h5JNIFatalError(env, "local c string is not pinned");                    \
        return (retdefault);                                                     \
    }                                                                            \
    (local2str) = ENVPTR->GetStringUTFChars(ENVPAR (java2str), &isCopy);         \
    if ((local2str) == NULL) {                                                   \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));             \
        h5JNIFatalError(env, "second local c string is not pinned");             \
        return (retdefault);                                                     \
    }                                                                            \
}

#define PIN_JAVA_STRING_TWO0(javastr,localstr,java2str,local2str) {              \
    jboolean isCopy;                                                             \
    if ((javastr) == NULL) {                                                     \
        h5nullArgument(env, "java string is NULL");                              \
        return;                                                                  \
    }                                                                            \
    if ((java2str) == NULL) {                                                    \
        h5nullArgument(env, "second java string is NULL");                       \
        return;                                                                  \
    }                                                                            \
    (localstr) = ENVPTR->GetStringUTFChars(ENVPAR (javastr), &isCopy);           \
    if ((localstr) == NULL) {                                                    \
        h5JNIFatalError(env, "local c string is not pinned");                    \
        return;                                                                  \
    }                                                                            \
    (local2str) = ENVPTR->GetStringUTFChars(ENVPAR (java2str), &isCopy);         \
    if ((local2str) == NULL) {                                                   \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));             \
        h5JNIFatalError(env, "second local c string is not pinned");             \
        return;                                                                  \
    }                                                                            \
}

#define UNPIN_JAVA_STRING_TWO(javastr,localstr,java2str,local2str) {             \
     ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));                \
     ENVPTR->ReleaseStringUTFChars(ENVPAR (java2str), (local2str));              \
}

#define PIN_JAVA_STRING_THREE(javastr,localstr,java2str,local2str,java3str,local3str,retdefault) {       \
    jboolean isCopy;                                                             \
    if ((javastr) == NULL) {                                                     \
        h5nullArgument(env, "java string is NULL");                              \
        return (retdefault);                                                     \
    }                                                                            \
    if ((java2str) == NULL) {                                                    \
        h5nullArgument(env, "second java string is NULL");                       \
        return (retdefault);                                                     \
    }                                                                            \
    if ((java3str) == NULL) {                                                    \
        h5nullArgument(env, "third java string is NULL");                        \
        return (retdefault);                                                     \
    }                                                                            \
    (localstr) = ENVPTR->GetStringUTFChars(ENVPAR (javastr), &isCopy);           \
    if ((localstr) == NULL) {                                                    \
        h5JNIFatalError(env, "local c string is not pinned");                    \
        return (retdefault);                                                     \
    }                                                                            \
    (local2str) = ENVPTR->GetStringUTFChars(ENVPAR (java2str), &isCopy);         \
    if ((local2str) == NULL) {                                                   \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));             \
        h5JNIFatalError(env, "second local c string is not pinned");             \
        return (retdefault);                                                     \
    }                                                                            \
    (local3str) = ENVPTR->GetStringUTFChars(ENVPAR (java3str), &isCopy);         \
    if ((local3str) == NULL) {                                                   \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));             \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (java2str), (local2str));           \
        h5JNIFatalError(env, "third local c string is not pinned");              \
        return (retdefault);                                                     \
    }                                                                            \
}

#define PIN_JAVA_STRING_THREE0(javastr,localstr,java2str,local2str,java3str,local3str) {       \
    jboolean isCopy;                                                             \
    if ((javastr) == NULL) {                                                     \
        h5nullArgument(env, "java string is NULL");                              \
        return;                                                                  \
    }                                                                            \
    if ((java2str) == NULL) {                                                    \
        h5nullArgument(env, "second java string is NULL");                       \
        return;                                                                  \
    }                                                                            \
    if ((java3str) == NULL) {                                                    \
        h5nullArgument(env, "third java string is NULL");                        \
        return;                                                                  \
    }                                                                            \
    (localstr) = ENVPTR->GetStringUTFChars(ENVPAR (javastr), &isCopy);           \
    if ((localstr) == NULL) {                                                    \
        h5JNIFatalError(env, "local c string is not pinned");                    \
        return;                                                                  \
    }                                                                            \
    (local2str) = ENVPTR->GetStringUTFChars(ENVPAR (java2str), &isCopy);         \
    if ((local2str) == NULL) {                                                   \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));             \
        h5JNIFatalError(env, "second local c string is not pinned");             \
        return;                                                                  \
    }                                                                            \
    (local3str) = ENVPTR->GetStringUTFChars(ENVPAR (java3str), &isCopy);         \
    if ((local3str) == NULL) {                                                   \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));             \
        ENVPTR->ReleaseStringUTFChars(ENVPAR (java2str), (local2str));           \
        h5JNIFatalError(env, "third local c string is not pinned");              \
        return;                                                                  \
    }                                                                            \
}

#define UNPIN_JAVA_STRING_THREE(javastr,localstr,java2str,local2str,java3str,local3str) {        \
     ENVPTR->ReleaseStringUTFChars(ENVPAR (javastr), (localstr));                \
     ENVPTR->ReleaseStringUTFChars(ENVPAR (java2str), (local2str));              \
     ENVPTR->ReleaseStringUTFChars(ENVPAR (java3str), (local3str));              \
}

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

extern jboolean h5JNIFatalError(JNIEnv *, const char *);
extern jboolean h5nullArgument(JNIEnv *, const char *);
extern jboolean h5badArgument (JNIEnv *, const char *);
extern jboolean h5outOfMemory (JNIEnv *, const char *);
extern jboolean h5libraryError(JNIEnv *env );
extern jboolean h5raiseException(JNIEnv *, const char *, const char *);
extern jboolean h5unimplemented( JNIEnv *env, const char *functName);

/* implemented at H5.c */
extern jint get_enum_value(JNIEnv *env, jobject enum_obj);
extern jobject get_enum_object(JNIEnv *env, const char* enum_class_name,
    jint enum_val, const char* enum_field_desc);

/* implemented at H5G.c */
extern jobject create_H5G_info_t(JNIEnv *env, H5G_info_t group_info);

#ifdef __cplusplus
} /* end extern "C" */
#endif /* __cplusplus */

#endif /* _Included_h5jni */
